% -------------------------------------------------------------------------------------------------------------------------
% THIS PROGRAM CONTAINS THE MATLAB CODES FOR SOLVING THE MODEL ISHOCK FOR THE PAPER
% "COMPARING MODELS OF MACROECONOMIC FLUCTUATIONS: HOW BIG ARE THE DIFFERENCES?" BY ANDRA GHENT (2008)
% -------------------------------------------------------------------------------------------------------------------------

function [IRFSneutral,IRFSgovt,IRFSishock]=RBCprior(params,n);

% Input is a vector of parameters and the number of period to run the
% impulse response function for
% Output is the impulse response functions for neutral technology shock

% Parametrization 
beta=params(1,:);  
theta=params(2,:);  
eta=params(3,:);
delta=params(4,:);                     
rhoA=params(5,:);
rhog=params(6,:);
rhoV=params(7,:);
g_y=params(8,:); 
sigA=0.0066;            
sigg=params(10,:);
sigg=0.0066;
sigV=params(11,:);
sigV=0.0066;
sigmeas=params(12,:);
                   

kappa = eta/beta-1+delta;
lambda=eta-1+delta;

% Relevant Matrices
A=[1 -theta*(lambda/kappa) 0; 1 0 theta-1; 1 0 -1];
B=[0 1-g_y-theta*(lambda/kappa); theta 0; 0 1];
C=[0 g_y 0; 1 0 0; 0 0 0];
D=[eta 0; kappa eta/beta];
F=[0 0 0; -kappa 0 0];
G=[(1-delta) 0; 0 eta/beta];
H=[0 lambda 0; 0 0 0];
J=[0 0 lambda; 0 0 (eta/beta-rhoV*(1-delta))];
Ainv=inv(A);
K=F*Ainv*C;
L=[K(1,1)*rhoA K(1,2)*rhog K(1,3)*rhoV; K(2,1)*rhoA K(2,2)*rhog K(2,3)*rhoV];
M=inv(D+F*Ainv*B)*(G+H*Ainv*B);
N=inv(D+F*Ainv*B)*(H*Ainv*C-L+J);

% Just need to check that M has one eigenvalue outside the unit circle and
% one inside so that the system has a unique solution

[Pinv,eigM]=eig(M);         
% We're all good

% The inversion between the intuitive definitions is just to be consistent
% with Ireland's work - again, see the notes to Ireland (2004) on Ireland's
% web page
Pinverse(:,1)=Pinv(:,2);
Pinverse(:,2)=Pinv(:,1);
P=inv(Pinverse);

P11=P(1,1);
P12=P(1,2);
P21=P(2,1);
P22=P(2,2);
N11=N(1,1);
N12=N(1,2);
N21=N(2,1);
N22=N(2,2);
lambda1M=eigM(2,2);
lambda2M=eigM(1,1);

R=P*N;

R11=R(1,1);
R12=R(1,2);
R13=R(1,3);
R21=R(2,1);
R22=R(2,2);
R23=R(2,3);

S1=-P21/P22;
S2=R21/(P22*(rhoA-lambda2M));
S3=R22/(P22*(rhog-lambda2M));
S4=R23/(P22*(rhoV-lambda2M));
S5=lambda1M;
S6=(P12*S2*(lambda1M-rhoA)+R11)/(P11+P12*S1);
S7=(P12*S3*(lambda1M-rhog)+R12)/(P11+P12*S1);
S8=(P12*S4*(lambda1M-rhoV)+R13)/(P11+P12*S1);
T1=Ainv*B*[1;S1];
T2=Ainv*C+Ainv*B*[0 0 0;S2 S3 S4];

% Final matrices for state space representation
% s(t+1)=PI*s(t)+W*eps(t)
% f(t)=U*s(t)
PI=[S5 S6 S7 S8; 0 rhoA 0 0; 0 0 rhog 0; 0 0 0 rhoV];
W=[0 0 0 ;1 0 0; 0 1 0; 0 0 1];
m1=(PI(1,1)*eta-(1-delta))/(eta-1+delta);
m2=PI(1,2)*eta/(eta-1+delta);
m3=PI(1,3)*eta/(eta-1+delta);
m4=PI(1,4)*eta/(eta-1+delta)-1;
m5=(1-g_y-theta*lambda/kappa)*S1+theta*(lambda/kappa)*m1;
m6=(1-g_y-theta*lambda/kappa)*S2+theta*(lambda/kappa)*m2;
m7=(1-g_y-theta*lambda/kappa)*S3+theta*(lambda/kappa)*m3+g_y;
m8=(1-g_y-theta*lambda/kappa)*S4+theta*(lambda/kappa)*m4;
U=[m5 m6 m7 m8; m1 m2 m3 m4; (m5-S1) (m6-S2) (m7-S3) (m8-S4); S1 S2 S3 S4];


% Now need to compute the IRFs

% First, look at the case for neutral technology shocks
% First, define the variables
irfend=40;
s = zeros(4,irfend);        % state vector for period 0 to irfend
                            % row entry denotes time period
                            % variable 1 is capital, variable 2 is
                            % TFP, variable 3 is g, variable 4 is V
sh=zeros(3,irfend);
sh(1,1)=sigA;

time=zeros(irfend,1);
for j=1:irfend
    time(j,1)=j;
end
f=zeros(4,irfend);             % vector of control variables
             % The order of the variables (4):  y i h c
s(:,1)=W*sh(:,1);                           
f(:,1)=U*s(:,1);
                           
for j = 2:irfend
    s(:,j)=PI*s(:,j-1)+W*sh(:,j);
    f(:,j)=U*s(:,j);
end
yneutralirf=f(1,1:irfend);
ineutralirf=f(2,1:irfend);
hneutralirf=f(3,1:irfend);
cneutralirf=f(4,1:irfend);
IRFSneutral=[yneutralirf' ineutralirf' hneutralirf' cneutralirf'];

% Now look at the case for government spending shocks
s = zeros(4,irfend);
sh=zeros(3,irfend);
sh(2,1)=sigg;

f=zeros(4,irfend);             % vector of control variables
             % The order of the variables (4):  y i h c
s(:,1)=W*sh(:,1);                           
f(:,1)=U*s(:,1);
                           
for j = 2:irfend
    s(:,j)=PI*s(:,j-1)+W*sh(:,j);
    f(:,j)=U*s(:,j);
end
ygovtirf=f(1,1:irfend);
igovtirf=f(2,1:irfend);
hgovtirf=f(3,1:irfend);
cgovtirf=f(4,1:irfend);

IRFSgovt=[ygovtirf' igovtirf' hgovtirf' cgovtirf'];

    
% Now look at the case for investment-specific technology shocks
sh=zeros(2,irfend);
s=zeros(4,irfend);
sh(3,1)=sigV;

s(:,1)=W*sh(:,1);
f(:,1)=U*s(:,1);

for j=2:irfend
    s(:,j)=PI*s(:,j-1)+W*sh(:,j);
    f(:,j)=U*s(:,j);
end
yinvestmentirf=f(1,1:irfend);
iinvestmentirf=f(2,1:irfend);
hinvestmentirf=f(3,1:irfend);
cinvestmentirf=f(4,1:irfend);

IRFSishock=[yinvestmentirf' iinvestmentirf' hinvestmentirf' cinvestmentirf'];

% Graph Impulse Response Functions
%figure('name','Figure 1: RBC','NumberTitle','off')
%subplot(221),plot(time,yneutralirf*100,time,ygovtirf*100,':',time,yinvestmentirf*100, '--'),title('Output'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(222),plot(time,ineutralirf*100,time,igovtirf*100,':',time,iinvestmentirf*100, '--'),title('Investment'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(223),plot(time,hneutralirf*100,time,hgovtirf*100,':',time,hinvestmentirf*100, '--'),title('Hours'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(224),plot(time,cneutralirf*100,time,cgovtirf*100,':',time,cinvestmentirf*100, '--'),title('Consumption'),xlabel('quarters'),ylabel('% dev. from steady state'),legend('Neutral Shock','Govt','I-Shock','Location','SouthEast')
        


% Now to get the parameters for the prior
sigepsilon=[sigA^2 0 0; 0 sigg^2 0; 0 0 sigV^2];
% Now solve for omegass
CC=W*sigepsilon*W';
OMEGAss=dlyap(PI,CC);

VARMEAS=[sigmeas^2 0 0 0; 0 0 0 0; 0 0 sigmeas^2 0; 0 0 0 0];

SIGyy=U*OMEGAss*U'+VARMEAS;

n=4;            % number of variables 
p=4;            % number of lags in the VAR
q=n*p+1;        % total number of variables in the VAR

SIGyx=zeros(n,q);
for k=1:p
    SIGyx(:,(k-1)*n+2:k*n+1)=U*PI^k*OMEGAss*U';
end

% NEED TO SET SIGxx MANUALLY!
% Here, it is done for a VAR with 4 lags
SIGxx=zeros(q,q);
SIGxx(1,1)=1;
for k=1:p
    SIGxx(2+(k-1)*n:1+k*n,2+(k-1)*n:1+k*n)=SIGyy;
end
SIGxx(2:5,6:9)=U*PI*OMEGAss*U';
SIGxx(2:5,10:13)=U*PI^2*OMEGAss*U';
SIGxx(2:5,14:17)=U*PI^3*OMEGAss*U';
SIGxx(6:9,2:5)=U*OMEGAss*PI'*U';
SIGxx(6:9,10:13)=U*PI*OMEGAss*U';
SIGxx(6:9,14:17)=U*PI^2*OMEGAss*U';
SIGxx(10:13,2:5)=U*OMEGAss*(PI^2)'*U';
SIGxx(10:13,6:9)=U*OMEGAss*PI'*U';
SIGxx(10:13,14:17)=U*PI*OMEGAss*U';
SIGxx(14:17,2:5)=U*OMEGAss*(PI^3)'*U';
SIGxx(14:17,6:9)=U*OMEGAss*(PI^2)'*U';
SIGxx(14:17,10:13)=U*OMEGAss*PI'*U';