% -------------------------------------------------------------------------------------------------------------------------
% THIS PROGRAM CONTAINS THE MATLAB CODES FOR SOLVING THE MODEL STICKY FOR THE PAPER
% "COMPARING MODELS OF MACROECONOMIC FLUCTUATIONS: HOW BIG ARE THE DIFFERENCES?" BY ANDRA GHENT (2008)
% -------------------------------------------------------------------------------------------------------------------------
function [GAMMAXX,GAMMAYX,GAMMAYY]=STICKYGAMMAs(x,X,Y);

% This program solves the sticky price model with an unaccommodating
% monetary authority

% Input is a vector of parameters and the number of period to run the
% impulse response function for
% Output is the impulse response functions for neutral technology shock

%% Other parameters
y0=X(1,14);%/x(2,1);
i0=X(1,15);%/x(2,1);
h0=X(1,16);%/x(2,1);
c0=X(1,17);%/x(2,1);

[T,n]=size(Y);
[T,k]=size(X);


% Parametrization 
beta=0.99;  
theta=x(1,:);  
eta=x(2,:);
delta=x(3,:);                     
rhoA=x(4,:);
rhog=x(5,:);
rhoV=x(6,:);
g_y=x(7,:);
sigA=x(8,:);            
sigg=x(9,:);
sigV=x(10,:);
sigmeas=x(11,:);
xi=x(12,:);
alpha=x(13,:);
q=x(14,:);

% Steady State
r=eta/beta-1+delta;
k_y=theta/r;
i_y=(eta-1+delta)*k_y;
c_y=1-g_y-(eta-1+delta)*k_y;
                        
a1=(1-alpha)*(1-alpha*beta)/(alpha*(1+beta-alpha*beta+alpha*beta*eta^(1-xi)));
a2=beta/(1+beta-alpha*beta+alpha*beta*eta^(1-xi));
a3=eta^(1-xi)/(1+beta-alpha*beta+alpha*beta*eta^(1-xi));

b1=a1*(1+theta*c_y/(1-theta));
b2=-a1*theta*(1+(theta+(1-delta)*k_y)/(1-theta));
b3=a2;
b4=a3;
b5=a1*theta*eta*k_y/(1-theta);
b6=-a1*(1+theta/(1-theta));
b7=a1*theta*g_y/(1-theta);
b8=a1*theta*(eta-1+delta)*k_y/(1-theta);

b9=1-r*c_y/((1+r-delta)*(1-theta));
b10=-(r*eta*k_y/((r+1-delta)*(1-theta))+eta^2*q/(r+1-delta));
b11=1+r/(r+1-delta);
b12=q*eta;
b13=-(r+1-delta)^(-1)*(r+q*eta^2+r*(theta+(1-delta)*k_y/(1-theta)))-q*eta;
b14=-r*rhoA/((r+1-delta)*(1-theta));
b15=r*g_y*rhog/((r+1-delta)*(1-theta));
b16=1-(1-delta)*rhoV/(r+1-delta)+(r/(r+1-delta))*((eta-1+delta)*k_y/(1-theta))*rhoV;

A=[0 0 0 beta beta; 0 0 0 -b4 0; 0 0 b10 0 b9; 1 0 0 0 0 ; 0 1 0 0 0];

B=[0 0 0 1 1; b3 b2 b5 -1 b1; 0 b12 b13 0 b11; 0 0 0 1 0; 0 0 1 0 0];

C=[0 0 0; b6 b7 b8; b14 b15 b16;0 0 0; 0 0 0];

D=inv(A)*B;
F=inv(A)*C;

[GinverseSP,H]=eigs(D);

% A necessary condition for a unique solution to exist, by Blanchard-Kahn
% (1980) Proposition 1, is that the number of non-predetermined vaiables is
% equal to the number of eigenvalues of D outside the unit circle
% here, this means we need 3 eigenvalues greater than 1

% We're good for all the calibrations I consider
Ginv=zeros(5,5);
Ginv(:,1)=GinverseSP(:,5);
Ginv(:,2)=GinverseSP(:,4);
Ginv(:,3)=GinverseSP(:,3);
Ginv(:,4)=GinverseSP(:,2);
Ginv(:,5)=GinverseSP(:,1);

G=inv(Ginv);

G11=G(1,1);
G12=G(1,2);
G13=G(1,3);
G14=G(1,4);
G15=G(1,5);
G21=G(2,1);
G22=G(2,2);
G23=G(2,3);
G24=G(2,4);
G25=G(2,5);
G31=G(3,1);
G32=G(3,2);
G33=G(3,3);
G34=G(3,4);
G35=G(3,5);
G41=G(4,1);
G42=G(4,2);
G43=G(4,3);
G44=G(4,4);
G45=G(4,5);
G51=G(5,1);
G52=G(5,2);
G53=G(5,3);
G54=G(5,4);
G55=G(5,5);

lambda1H=H(5,5);
lambda2H=H(4,4);
lambda3H=H(3,3);
lambda4H=H(2,2);
lambda5H=H(1,1);

J=G*F;

J11=J(1,1);
J12=J(1,2);
J13=J(1,3);
J21=J(2,1);
J22=J(2,2);
J23=J(2,3);
J31=J(3,1);
J32=J(3,2);
J33=J(3,3);
J41=J(4,1);
J42=J(4,2);
J43=J(4,3);
J51=J(5,1);
J52=J(5,2);
J53=J(5,3);

K=G(3:5,3:5);
L=-G(3:5,1:2);
M=[J31/(rhoA-lambda3H) J32/(rhog-lambda3H) J33/(rhoV-lambda3H); J41/(rhoA-lambda4H) J42/(rhog-lambda4H) J43/(rhoV-lambda4H); J51/(rhoA-lambda5H) J52/(rhog-lambda5H) J53/(rhoV-lambda5H)];
N=inv(K)*L;
O=inv(K)*M;

P=zeros(5,5);
P(1,1)=G11+G13*N(1,1)+G14*N(2,1)+G15*N(3,1);
P(1,2)=G12+G13*N(1,2)+G14*N(2,2)+G15*N(3,2);
P(2,1)=G21+G23*N(1,1)+G24*N(2,1)+G25*N(3,1);
P(2,2)=G22+G23*N(1,2)+G24*N(2,2)+G25*N(3,2);
P(3,3)=1;
P(4,4)=1;
P(5,5)=1;

Q=zeros(5,5);
Q(1,1)=lambda1H*P(1,1);
Q(1,2)=lambda1H*P(1,2);
Q(1,3)=J11+(lambda1H-rhoA)*(G13*O(1,1)+G14*O(2,1)+G15*O(3,1));
Q(1,4)=J12+(lambda1H-rhog)*(G13*O(1,2)+G14*O(2,2)+G15*O(3,2));
Q(1,5)=J13+(lambda1H-rhoV)*(G13*O(1,3)+G14*O(2,3)+G15*O(3,3));
Q(2,1)=lambda2H*P(2,1);
Q(2,2)=lambda2H*P(2,2);
Q(2,3)=J21+(lambda2H-rhoA)*(G23*O(1,1)+G24*O(2,1)+G25*O(3,1));
Q(2,4)=J22+(lambda2H-rhog)*(G23*O(1,2)+G24*O(2,2)+G25*O(3,2));
Q(2,5)=J23+(lambda2H-rhoV)*(G23*O(1,3)+G24*O(2,3)+G25*O(3,3));
Q(3,3)=rhoA;
Q(4,4)=rhog;
Q(5,5)=rhoV;

PI=inv(P)*Q;

W=[0 0 0; 0 0 0; 1 0 0; 0 1 0; 0 0 1];

t1=c_y*N(3,1)/(1-theta)+eta*k_y*PI(2,1)/(1-theta);
t2=c_y*N(3,2)/(1-theta)+eta*k_y*PI(2,2)/(1-theta)-(theta+(1-delta)*k_y)/(1-theta);
t3=c_y*O(3,1)/(1-theta)+eta*k_y*PI(2,3)/(1-theta)-1/(1-theta);
t4=c_y*O(3,2)/(1-theta)+eta*k_y*PI(2,4)/(1-theta)+g_y/(1-theta);
t5=c_y*O(3,3)/(1-theta)+eta*k_y*PI(2,5)/(1-theta)+(eta-1+delta)*k_y/(1-theta);

t6=eta*PI(2,1)/(eta-1+delta);
t7=(eta*PI(2,2)-1+delta)/(eta-1+delta);
t8=eta*PI(2,3)/(eta-1+delta);
t9=eta*PI(2,4)/(eta-1+delta);
t10=eta*PI(2,5)/(eta-1+delta)-1;

U=[(1-theta)*t1 (theta+(1-theta)*t2) ((1-theta)*t3+1) (1-theta)*t4 (1-theta)*t5; t6 t7 t8 t9 t10; t1 t2 t3 t4 t5; N(3,1) N(3,2) O(3,1) O(3,2) O(3,3)];


% Now solve for omegass
sigepsilon=[sigA^2 0 0; 0 sigg^2 0; 0 0 sigV^2];
CC=W*sigepsilon*W';
OMEGAss=dlyap(PI,CC);

VARMEAS=[sigmeas^2 0 0 0; 0 0 0 0; 0 0 0 0; 0 0 0 0];

SIGyy=U*OMEGAss*U'+VARMEAS;

n=4;            % number of variables 
p=4;            % number of lags in the VAR
q=n*p+1;        % total number of variables in the VAR

% Creating a matrix that will keep take into account the trend in E(Y*'*Y*)
etasq=0;
etaonh=0;
sumTs=0;
for j=1:T
    etasq=etasq+eta^(2*(p+j));
    etaonh=etaonh+eta^(p+j);
    sumTs=sumTs+1;
end
SIGyytrend=etasq*ones(n,n);
SIGyytrend(1,3)=etaonh;
SIGyytrend(2,3)=etaonh;
SIGyytrend(3,1)=etaonh;
SIGyytrend(3,2)=etaonh;
SIGyytrend(3,3)=sumTs;
SIGyytrend(3,4)=etaonh;
SIGyytrend(4,3)=etaonh;
SIGyytrend;

% Creating a matrix that will take into account initial conditions
SIGyyinit=zeros(n,n);
SIGyyinit(1,1)=y0^2;
SIGyyinit(1,2)=y0*i0;
SIGyyinit(1,3)=y0*h0;
SIGyyinit(1,4)=y0*c0;
SIGyyinit(2,1)=i0*y0;
SIGyyinit(2,2)=i0^2;
SIGyyinit(2,3)=i0*h0;
SIGyyinit(2,4)=i0*c0;
SIGyyinit(3,1)=h0*y0;
SIGyyinit(3,2)=h0*i0;
SIGyyinit(3,3)=h0^2;
SIGyyinit(3,4)=h0*c0;
SIGyyinit(4,1)=c0*y0;
SIGyyinit(4,2)=c0*i0;
SIGyyinit(4,3)=c0*h0;
SIGyyinit(4,4)=c0^2;

GAMMAYY=T^(-1)*SIGyytrend.*(SIGyy+SIGyyinit);

SIGyx=zeros(n,q);
for kk=1:p
    SIGyx(:,(kk-1)*n+2:kk*n+1)=U*PI^kk*OMEGAss*U';
end
SIGyxtrend=zeros(n,q);
SIGyxtrend(1,1)=etaonh;
SIGyxtrend(2,1)=etaonh;
SIGyxtrend(3,1)=sumTs;
SIGyxtrend(4,1)=etaonh;
SIGyxtrend(1,2)=etasq/eta;
SIGyxtrend(1,3)=etasq/eta;
SIGyxtrend(1,4)=etaonh;
SIGyxtrend(1,5)=etasq/eta;
SIGyxtrend(1,6)=etasq/eta^2;
SIGyxtrend(1,7)=etasq/eta^2;
SIGyxtrend(1,8)=etaonh;
SIGyxtrend(1,9)=etasq/eta^2;
SIGyxtrend(1,10)=etasq/eta^3;
SIGyxtrend(1,11)=etasq/eta^3;
SIGyxtrend(1,12)=etaonh;
SIGyxtrend(1,13)=etasq/eta^3;
SIGyxtrend(1,14)=etasq/eta^4;
SIGyxtrend(1,15)=etasq/eta^4;
SIGyxtrend(1,16)=etaonh;
SIGyxtrend(1,17)=etasq/eta^4;
SIGyxtrend(2,2:17)=SIGyxtrend(1,2:17);
SIGyxtrend(3,2)=etaonh/eta;
SIGyxtrend(3,3)=etaonh/eta;
SIGyxtrend(3,4)=sumTs;
SIGyxtrend(3,5)=etaonh/eta;
SIGyxtrend(3,6)=etaonh/eta^2;
SIGyxtrend(3,7)=etaonh/eta^2;
SIGyxtrend(3,8)=sumTs;
SIGyxtrend(3,9)=etaonh/eta^2;
SIGyxtrend(3,10)=etaonh/eta^3;
SIGyxtrend(3,11)=etaonh/eta^3;
SIGyxtrend(3,12)=sumTs;
SIGyxtrend(3,13)=etaonh/eta^3;
SIGyxtrend(3,14)=etaonh/eta^4;
SIGyxtrend(3,15)=etaonh/eta^4;
SIGyxtrend(3,16)=sumTs;
SIGyxtrend(3,17)=etaonh/eta^4;
SIGyxtrend(4,2:17)=SIGyxtrend(1,2:17);
SIGyxtrend;

SIGyxinit=zeros(n,q);
SIGyxinit(1:4,1)=[y0 i0 h0 c0];
SIGyxinit(1:4,2:5)=[y0^2 y0*i0 y0*h0 y0*c0;y0*i0 i0^2 i0*h0 i0*c0;...
    h0*y0 h0*i0 h0^2 h0*c0; c0*y0 c0*i0 c0*h0 c0^2];
SIGyxinit(1:4,6:9)=SIGyxinit(:,2:5);
SIGyxinit(:,10:13)=SIGyxinit(:,2:5);
SIGyxinit(:,14:17)=SIGyxinit(:,2:5);
SIGyxtrend.*SIGyxinit;

GAMMAYX=T^(-1)*SIGyxtrend.*(SIGyx + SIGyxinit);
    

SIGxx=zeros(q,q);
for kk=1:p
    SIGxx(2+(kk-1)*n:1+kk*n,2+(kk-1)*n:1+kk*n)=SIGyy;
end
SIGxx(2:5,6:9)=U*PI*OMEGAss*U';
SIGxx(2:5,10:13)=U*PI^2*OMEGAss*U';
SIGxx(2:5,14:17)=U*PI^3*OMEGAss*U';
SIGxx(6:9,2:5)=U*OMEGAss*PI'*U';
SIGxx(6:9,10:13)=U*PI*OMEGAss*U';
SIGxx(6:9,14:17)=U*PI^2*OMEGAss*U';
SIGxx(10:13,2:5)=U*OMEGAss*(PI^2)'*U';
SIGxx(10:13,6:9)=U*OMEGAss*PI'*U';
SIGxx(10:13,14:17)=U*PI*OMEGAss*U';
SIGxx(14:17,2:5)=U*OMEGAss*(PI^3)'*U';
SIGxx(14:17,6:9)=U*OMEGAss*(PI^2)'*U';
SIGxx(14:17,10:13)=U*OMEGAss*PI'*U';

SIGxxtrend=zeros(q,q);
SIGxxtrend(1,1)=sumTs;
SIGxxtrend(1,2)=etaonh/eta;
SIGxxtrend(1,3)=etaonh/eta;
SIGxxtrend(1,4)=sumTs;
SIGxxtrend(1,5)=etaonh/eta;
SIGxxtrend(1,6)=etaonh/eta^2;
SIGxxtrend(1,7)=etaonh/eta^2;
SIGxxtrend(1,8)=sumTs;
SIGxxtrend(1,9)=etaonh/eta^2;
SIGxxtrend(1,10)=etaonh/eta^3;
SIGxxtrend(1,11)=etaonh/eta^3;
SIGxxtrend(1,12)=sumTs;
SIGxxtrend(1,13)=etaonh/eta^3;
SIGxxtrend(1,14)=etaonh/eta^4;
SIGxxtrend(1,15)=etaonh/eta^4;
SIGxxtrend(1,16)=sumTs;
SIGxxtrend(1,17)=etaonh/eta^4;
SIGxxtrend(2,1)=etaonh/eta;
SIGxxtrend(2,2)=etasq/eta^2;
SIGxxtrend(2,3)=etasq/eta^2;
SIGxxtrend(2,4)=etaonh/eta;
SIGxxtrend(2,5)=etasq/eta^2;
SIGxxtrend(2,6:9)=SIGxxtrend(2,2:5)./eta;
SIGxxtrend(2,10:13)=SIGxxtrend(2,2:5)./eta^2;
SIGxxtrend(2,14:17)=SIGxxtrend(2,2:5)/eta^3;
SIGxxtrend(3,:)=SIGxxtrend(2,:);
SIGxxtrend(4,1)=sumTs;
SIGxxtrend(4,2)=etaonh/eta;
SIGxxtrend(4,3)=etaonh/eta;
SIGxxtrend(4,4)=sumTs;
SIGxxtrend(4,5)=etaonh/eta;
SIGxxtrend(4,6)=etaonh/eta^2;
SIGxxtrend(4,7)=etaonh/eta^2;
SIGxxtrend(4,8)=sumTs;
SIGxxtrend(4,9)=etaonh/eta^2;
SIGxxtrend(4,10)=etaonh/eta^3;
SIGxxtrend(4,11)=etaonh/eta^3;
SIGxxtrend(4,12)=sumTs;
SIGxxtrend(4,13)=etaonh/eta^3;
SIGxxtrend(4,14)=etaonh/eta^4;
SIGxxtrend(4,15)=etaonh/eta^4;
SIGxxtrend(4,16)=sumTs;
SIGxxtrend(4,17)=etaonh/eta^4;
SIGxxtrend(5,:)=SIGxxtrend(2,:);
SIGxxtrend(6,:)=SIGxxtrend(2,:)./eta;
SIGxxtrend(7,:)=SIGxxtrend(3,:)./eta;
SIGxxtrend(8,:)=SIGxxtrend(4,:)./eta;
SIGxxtrend(8,1)=sumTs;
SIGxxtrend(8,4)=sumTs;
SIGxxtrend(8,8)=sumTs;
SIGxxtrend(8,12)=sumTs;
SIGxxtrend(8,16)=sumTs;
SIGxxtrend(9,:)=SIGxxtrend(5,:)./eta;
SIGxxtrend(10,:)=SIGxxtrend(6,:)./eta;
SIGxxtrend(11,:)=SIGxxtrend(7,:)./eta;
SIGxxtrend(12,:)=SIGxxtrend(8,:)./eta;
SIGxxtrend(12,1)=sumTs;
SIGxxtrend(12,4)=sumTs;
SIGxxtrend(12,8)=sumTs;
SIGxxtrend(12,12)=sumTs;
SIGxxtrend(12,16)=sumTs;
SIGxxtrend(13,:)=SIGxxtrend(9,:)./eta;
SIGxxtrend(14,:)=SIGxxtrend(10,:)./eta;
SIGxxtrend(15,:)=SIGxxtrend(11,:)./eta;
SIGxxtrend(16,:)=SIGxxtrend(12,:)./eta;
SIGxxtrend(16,1)=sumTs;
SIGxxtrend(16,4)=sumTs;
SIGxxtrend(16,8)=sumTs;
SIGxxtrend(16,12)=sumTs;
SIGxxtrend(16,16)=sumTs;
SIGxxtrend(17,:)=SIGxxtrend(13,:)./eta;

iota=ones(1,4);
init=[y0;i0;h0;c0];
initsq=[y0^2 y0*i0 y0*h0 y0*c0; i0*y0 i0^2 i0*h0 i0*c0; h0*y0 h0*i0 h0^2 h0*c0; c0*y0 c0*i0 c0*h0 c0^2];
SIGxxinit=zeros(q,q);
SIGxxinit(:,1)=[1; reshape(kron(init,iota),16,1)];
SIGxxinit(1,:)=SIGxxinit(:,1)';
SIGxxinit(2:17,2:17)=[repmat(initsq,4)];

GAMMAXX=T^(-1)*SIGxxtrend.*(SIGxx+SIGxxinit);